;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; **************** Bridge control macros ***************************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Enable bridge A control
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro BRIDGE_A_ENABLE
	SD1A_SET
	SD2A_SET
	; Check which way the bridge should be activated
	; Uses global variable CH_A_B_PHASE to select path
	sbrs CH_A_B_PHASE,CH_A_PATH_1_BIT
	rjmp bridge_a_set_path_2
	; Enable bridge A path 1
	H1A_SET
	H2A_CLR
	rjmp bridge_a_enable_end
bridge_a_set_path_2:
	; Enable bridge A path 2
	H1A_CLR
	H2A_SET
bridge_a_enable_end:
.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Enable bridge B control
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro BRIDGE_B_ENABLE
	SD1B_SET
	SD2B_SET
	; Check which way the bridge should be activated
	sbrs CH_A_B_PHASE,CH_B_PATH_1_BIT
	rjmp bridge_b_set_path_2
	; Enable bridge A path 1
	H1B_SET
	H2B_CLR
	rjmp bridge_b_enable_end
bridge_b_set_path_2:
	; Enable bridge A path 2
	H1B_CLR
	H2B_SET
bridge_b_enable_end:
.endmacro 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Disable bridge A control (fast decay mode)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro BRIDGE_A_DISABLE_FAST
	; Fast mode - disable all A bridge drivers
	SD1A_CLR
	SD2A_CLR
.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Disable bridge B control (fast decay mode)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro BRIDGE_B_DISABLE_FAST
	; Fast mode - disable all B bridge drivers
	SD1B_CLR
	SD2B_CLR
.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Disable bridge A control (slow decay mode)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro BRIDGE_A_DISABLE_SLOW
	; Slow mode - disable source drivers only
	; Uses global variable CH_A_B_PHASE to select path
	sbrs CH_A_B_PHASE,CH_A_PATH_1_BIT
	rjmp bridge_a_disable_path_2
	SD1A_CLR
	SD2A_SET
	rjmp bridge_a_disable_end
bridge_a_disable_path_2:
	SD1A_SET
	SD2A_CLR
bridge_a_disable_end:	
.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Disable bridge B control (slow decay mode)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro BRIDGE_B_DISABLE_SLOW
	; Slow mode - disable source drivers only
	sbrs CH_A_B_PHASE,CH_B_PATH_1_BIT
	rjmp bridge_b_disable_path_2
	SD1B_CLR
	SD2B_SET
	rjmp bridge_b_disable_end
bridge_b_disable_path_2:
	SD1B_SET
	SD2B_CLR
bridge_b_disable_end:	
.endmacro



