;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; **************** Current reduction routines***********************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Restart of current reduction timer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CLEAR_CURR_REDUCTION_TIMEOUT
	clr CURR_LIMIT_TOUT_L
	clr CURR_LIMIT_TOUT_H 
.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Initialization of current reduction timer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CURR_REDUCTION_INIT
	CLEAR_CURR_REDUCTION_TIMEOUT
.endmacro



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Main current reduction process
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CHECK_CURR_REDUCTION

	cpi IDLE_STATUS,0
	breq no_current_reduction	; no current reduction
	cpi IDLE_STATUS,1
	brne already_reduced		; current already reduced
	ser  IDLE_STATUS			; mark current as reduced

	; Reduce the current to 50%

	; Channel A	
	ldi temp,CH_A_SPI_H_DATA_MASK
	and CH_A_SPI_H,temp  			; isolate hi byte data
	clc
	ror CH_A_SPI_H					; divide hi/lo pair by 2
	ror CH_A_SPI_L
	ldi temp,CH_A_SPI_H_CTRL		; add again control word
	or  CH_A_SPI_H,temp 
 

	; Channel B
	ldi temp,CH_B_SPI_H_DATA_MASK
	and CH_B_SPI_H,temp  			; isolate hi byte data
	clc
	ror CH_B_SPI_H					; divide hi/lo pair by 2
	ror CH_B_SPI_L
	ldi temp,CH_B_SPI_H_CTRL		; add again control word
	or  CH_B_SPI_H,temp 

	ser temp						; notify about the change
									; D/A should be updated

no_current_reduction:
	
	; increment timeout counter
	ldi temp,1
	add CURR_LIMIT_TOUT_L,temp
	clr temp
	adc CURR_LIMIT_TOUT_H,temp
	; check for timer timeout
	ldi temp,CURR_LIMIT_TOUT_H_TIMEOUT
	cp  CURR_LIMIT_TOUT_H,temp
	brne no_timeout
	; timeout, we have to reduce current next time we enter the procedure
	ldi IDLE_STATUS,1

already_reduced:
no_timeout:
	clr temp
.endmacro


