;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; **************** Decay control  ***************************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



#define  BIT_A_OFF_MODE			0
#define  BIT_A_OFF_SLOW_MODE	1

#define  BIT_B_OFF_MODE			4
#define  BIT_B_OFF_SLOW_MODE	5


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Some private macros for timer register manipulation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CMP_A_TIMER_BLINK_SETUP
	ldi  temp,DECAY_BLINK_TOUT_VAL
	out  OCR0A,temp
.endmacro

.macro CMP_A_TIMER_OFF_SETUP
	ldi  temp,DECAY_OFF_TOUT_VAL
	out  OCR0B,temp
.endmacro

.macro CMP_A_TIMER_FAST_SETUP
	ldi  temp,DECAY_FAST_TOUT_VAL
	out  OCR0A,temp
.endmacro


.macro CMP_B_TIMER_BLINK_SETUP
	ldi  temp,DECAY_BLINK_TOUT_VAL
	sts  OCR2A,temp
.endmacro

.macro CMP_B_TIMER_OFF_SETUP
	ldi  temp,DECAY_OFF_TOUT_VAL
	sts  OCR2B,temp
.endmacro

.macro CMP_B_TIMER_FAST_SETUP
	ldi  temp,DECAY_FAST_TOUT_VAL
	sts  OCR2A,temp
.endmacro



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Initialization of decay control timers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CMP_DECAY_INIT

	; Initialize timers used for devay control


	; Timer 0 for bridge A	
	ldi  temp, 0
	out  TCCR0A,temp	; no output control
	ldi  temp,_BV(CS01) 
	out  TCCR0B,temp	; Normal mode, clk/8


	CMP_A_TIMER_BLINK_SETUP
	CMP_A_TIMER_OFF_SETUP
		
	; Timer 2 for bridge B
	
	ldi  temp, 0
	sts  TCCR2A,temp	; no output control	
	ldi  temp,_BV(CS21) 
	sts  TCCR2B,temp	; Normal mode, clk/8

	CMP_B_TIMER_BLINK_SETUP
	CMP_B_TIMER_OFF_SETUP

.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Reset of decay control timers for bridge A
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CMP_A_CLEAR_TIMERS

	; clear timer 0
	clr  temp
	out  TCNT0,temp
	sbi  TIFR0,OCF0A
	sbi  TIFR0,OCF0B

.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Reset of decay control timers for bridge B
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CMP_B_CLEAR_TIMERS

	; clear timer 2
	clr  temp
	sts  TCNT2,temp
	sbi  TIFR2,OCF2A
	sbi  TIFR2,OCF2B
	

.endmacro



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Reset of decay control for both bridges 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro CMP_DECAY_PREPARE_NEXT


	CMP_A_TIMER_BLINK_SETUP
	CMP_B_TIMER_BLINK_SETUP

	CMP_A_CLEAR_TIMERS
	CMP_B_CLEAR_TIMERS

	clr DECAY_STATUS 


.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Bridge A control for microstep mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CMP_DECAY_CHECK_A
	sbis TIFR0,OCF0A
	rjmp check_cmp_A_end	 	; still in blink/fast timeout

	sbrc DECAY_STATUS, BIT_A_OFF_MODE
	rjmp bridge_in_off_mode		

	; Check CMP_A signal - if overcurrent then disable control
	SBIC_CMP_A	
	rjmp check_cmp_A_end		; no overcurrent, jump out

	; Overcurrent mode
overcurrent_mode:
	BRIDGE_A_DISABLE_FAST		; overcurrent - disable control of CH_A
	CMP_A_TIMER_FAST_SETUP		; setbridge in fast decay mode
	CMP_A_CLEAR_TIMERS
	sbr DECAY_STATUS, _BV(BIT_A_OFF_MODE)
	rjmp check_cmp_A_end

bridge_in_off_mode:
;	sbrc DECAY_STATUS, BIT_A_OFF_SLOW_MODE
;	rjmp check_off_timeout		; bridge already in slow decay mode
;	sbr DECAY_STATUS, _BV(BIT_A_OFF_SLOW_MODE)
;	BRIDGE_A_DISABLE_SLOW
;	rjmp check_cmp_A_end	 	
;
;check_off_timeout:
	sbis TIFR0,OCF0B
	rjmp check_cmp_A_end		; still off mode
reenable_bridge:
	BRIDGE_A_ENABLE
	cbr DECAY_STATUS, _BV(BIT_A_OFF_MODE) | _BV(BIT_A_OFF_SLOW_MODE)
	CMP_A_CLEAR_TIMERS
	CMP_A_TIMER_BLINK_SETUP		; blink activated when we enable bridge again
check_cmp_A_end:

.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Bridge B control for microstep mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CMP_DECAY_CHECK_B
	
	sbis TIFR2,OCF2A
	rjmp check_cmp_B_end	 	; still in blink/fast timeout

	sbrc DECAY_STATUS, BIT_B_OFF_MODE
	rjmp bridge_in_off_mode		

	; Check CMP_A signal - if overcurrent then disable control
	SBIC_CMP_B
	rjmp check_cmp_B_end		; no overcurrent, jump out

	; Overcurrent mode
overcurrent_mode:
	BRIDGE_B_DISABLE_FAST		; overcurrent - disable control of CH_B
	CMP_B_TIMER_FAST_SETUP		; setbridge in fast decay mode
	CMP_B_CLEAR_TIMERS
	sbr DECAY_STATUS, _BV(BIT_B_OFF_MODE)
	rjmp check_cmp_B_end

bridge_in_off_mode:
;	sbrc DECAY_STATUS, BIT_B_OFF_SLOW_MODE
;	rjmp check_off_timeout		; bridge already in slow decay mode
;	sbr DECAY_STATUS, _BV(BIT_B_OFF_SLOW_MODE)
;	BRIDGE_B_DISABLE_SLOW
;	rjmp check_cmp_B_end	 	
;
;check_off_timeout:
	sbis TIFR2,OCF2B
	rjmp check_cmp_B_end		; still off mode
reenable_bridge:
	BRIDGE_B_ENABLE
	cbr DECAY_STATUS, _BV(BIT_B_OFF_MODE) | _BV(BIT_B_OFF_SLOW_MODE)
	CMP_B_CLEAR_TIMERS
	CMP_B_TIMER_BLINK_SETUP		; blink activated when we enable bridge again
check_cmp_B_end:

.endmacro








