;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ***************  Pin control mapping macros ***********************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Pin mapping
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; DIR pin	
.macro SBIC_DIR
	sbic PIND,6
.endmacro

.macro CFG_DIR
	cbi DDRD,6
.endmacro


; CLK pin	
.macro SBIC_CLK	 
	sbic PIND,5
.endmacro

.macro CFG_CLK
	cbi DDRD,5
.endmacro


;  ENABLE pin
.macro SBIC_ENABLE 
	sbic PIND,7
.endmacro

.macro ENABLE_CLK
	cbi DDRD,7
.endmacro


.macro CFG_ENABLE 
	cbi DDRB,7
.endmacro

;  MISC pin
.macro SBIC_MISC
	sbic PINB,1
.endmacro

.macro CFG_MISC
	cbi DDRB,1
.endmacro


;  JUMPER
.macro SBIC_JUMPER
	sbic PINB,0
.endmacro

.macro CFG_JUMPER
	cbi DDRB,0
.endmacro


;  CMP_A pin
.macro SBIC_CMP_A	 
	sbic PINC,0
.endmacro

.macro CFG_CMP_A
	cbi DDRC,0
.endmacro

;  CMP_B pin
.macro SBIC_CMP_B  
	sbic PINC,1
.endmacro

.macro CFG_CMP_B
	cbi DDRC,1
.endmacro


;  LOAD pin
.macro LOAD_SET
	sbi PORTD,4
.endmacro

.macro LOAD_CLR
	cbi PORTD,4
.endmacro

.macro CFG_LOAD
	sbi DDRD,4
.endmacro



;  LED pin
.macro LED_SET
	sbi PORTB,2
.endmacro

.macro LED_TOGGLE
	in  temp,PORTB
	ldi temp2,_BV(2)
	eor temp,temp2
	out PORTB,temp
.endmacro

.macro LED_CLR
	cbi PORTB,2
.endmacro


.macro CFG_LED
	sbi DDRB,2
.endmacro


;  H1B pin
.macro H1B_SET
	sbi PORTC,5
.endmacro

.macro H1B_CLR
	cbi PORTC,5
.endmacro

.macro CFG_H1B
	sbi DDRC,5
.endmacro

	
;  H2B pin
.macro H2B_SET
	sbi PORTC,3
.endmacro

.macro H2B_CLR
	cbi PORTC,3
.endmacro

.macro CFG_H2B
	sbi DDRC,3
.endmacro


;  H1A	pin
.macro H1A_SET
	sbi PORTD,0
.endmacro

.macro H1A_CLR
	cbi PORTD,0
.endmacro

.macro CFG_H1A
	sbi DDRD,0
.endmacro


;  H2A	pin
.macro H2A_SET
	sbi PORTD,2
.endmacro

.macro H2A_CLR
	cbi PORTD,2
.endmacro

.macro CFG_H2A
	sbi DDRD,2
.endmacro


;  SD2A pin
.macro SD2A_SET
	sbi PORTD,3
.endmacro

.macro SD2A_CLR
	cbi PORTD,3
.endmacro

.macro CFG_SD2A
	sbi DDRD,3
.endmacro


;  SD1A pin
.macro SD1A_SET
	sbi PORTD,1
.endmacro

.macro SD1A_CLR
	cbi PORTD,1
.endmacro

.macro CFG_SD1A
	sbi DDRD,1
.endmacro


;  SD2B	pin
.macro SD2B_SET
	sbi PORTC,2
.endmacro

.macro SD2B_CLR
	cbi PORTC,2
.endmacro


.macro CFG_SD2B
	sbi DDRC,2
.endmacro


;  SD1B pin
.macro SD1B_SET
	sbi PORTC,4
.endmacro

.macro SD1B_CLR
	cbi PORTC,4
.endmacro

.macro CFG_SD1B
	sbi DDRC,4
.endmacro

;  SPI pins

.macro CFG_SPI_PINS
	sbi DDRB,3	; MOSI output
	sbi DDRB,5	; SCK  output
	cbi	DDRB,4  ; MISO input 
	; Note - SS ( PORTB.2) must be configured as output
	;        Otherwise SPI will not work properly
	sbi PORTB,2
.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Configure I/O registers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CONFIGURE_IO_PORTS

	; Configure pins

	CFG_SD1B
	CFG_SD2B
	CFG_SD1A
	CFG_SD2A
	CFG_H1A
	CFG_H2A
	CFG_H1B
	CFG_H2B
	CFG_LED
	CFG_LOAD
	CFG_CMP_A
	CFG_CMP_B
	CFG_JUMPER
	CFG_MISC
	CFG_DIR
	CFG_CLK
	CFG_ENABLE
	CFG_SPI_PINS

	; Disable globally pull-ups
	in  temp, MCUCR
	sbr temp, _BV(PUD)
	out MCUCR,temp

.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ***************  Pin control process macros ***********************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ENABLE signal processing
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro CHECK_ENABLE

	SBIC_ENABLE
	rjmp check_enabled 			; control enabled - continue
	clr temp
	rjmp check_enable_end 		; we wait until control is enabled	
check_enabled:
	ser temp
check_enable_end:
.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; CLK signal processing
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro	CHECK_STEP

	; Check whether to make next step
	clr  temp
	SBIC_CLK
	rjmp check_step_high		; high state of PC_CLK
	ldi  STEP_STATUS,0		 	; low state of PC_CLK 
	rjmp check_step_end

check_step_high:

	sbrc STEP_STATUS,0 
	rjmp check_step_end			;  step already processed
	ldi  STEP_STATUS,1			;  step not yet processed
	ser  temp 
check_step_end:
.endmacro

