;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; **************** Step control macros   ***************************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Few constants for embedded half step operation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


#define SIN_00  0x000 ; 0 of 4 (+0.0000 (+0.00 deg)sinus value  +0.00 volt )
#define SIN_45  0xb50 ; 1 of 4 (+0.7071 (+45.00 deg)sinus value  +2.90 volt )
#define SIN_90  0xfff ; 2 of 4 (+1.0000 (+90.00 deg)sinus value  +4.09 volt )

#define STEP_0 0x80|0x00|0x20|0x00  ; (A_P1/B_P2/A_SLOW /B_MIXED) 0 of 8 
#define STEP_1 0x80|0x00|0x20|0x00  ; (A_P1/B_P2/A_SLOW /B_MIXED) 1 of 8 
#define STEP_2 0x80|0x40|0x00|0x10  ; (A_P1/B_P1/A_MIXED/B_SLOW ) 2 of 8 
#define STEP_3 0x80|0x40|0x00|0x10  ; (A_P1/B_P1/A_MIXED/B_SLOW ) 3 of 8 
#define STEP_4 0x00|0x40|0x20|0x00  ; (A_P2/B_P1/A_SLOW /B_MIXED) 4 of 8 
#define STEP_5 0x00|0x40|0x20|0x00  ; (A_P2/B_P1/A_SLOW /B_MIXED) 5 of 8 
#define STEP_6 0x00|0x00|0x00|0x10  ; (A_P2/B_P2/A_MIXED/B_SLOW ) 6 of 8 
#define STEP_7 0x00|0x00|0x00|0x10  ; (A_P2/B_P2/A_MIXED/B_SLOW ) 7 of 8 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Step module intialization
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.macro OVERSTEP_INIT
	
	clr OVERSTEP_HALFSTEP_INDEX			     
	clr OVERSTEP_DIV_CNTR		
	clr OVERSTEP_MUL_CNTR	
	clr OVER_TIMEOUT_CNTR_UP
	clr OVER_TIMEOUT_CNTR_DN
	clr STEP_PERIOD

	ldi STEP_MODE,STEP_MULTIPLIED_MODE
	
	; Configure Timer 1
	; No output control
	ldi  temp, 0
	sts  TCCR1A,temp
	; CTC mode, clk/1
	ldi  temp,_BV(WGM12)|_BV(CS10)
	sts  TCCR1B,temp
	; Setup timeout
	ldi temp, high( (STEP_MULTIPLIED_TIMEOUT * (HW_MCK /1 )) / 100000 )
	ldi temp2,low( (STEP_MULTIPLIED_TIMEOUT * (HW_MCK /1 )) / 100000 )
	sts  OCR1AH,temp
	sts  OCR1AL,temp2

	ldi  temp, _BV(OCF1A)
	sts  TIMSK1,temp
				
.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Macro for counting real step time period
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro OVERSTEP_COUNT_FREQ
	sbis TIFR1,OCF1A
	rjmp no_timeout	; no current reduction

	; Ok timeout has occured, clear the timeout flag
	sbi  TIFR1,OCF1A
	ldi  temp,0xff
	cp   STEP_PERIOD,temp
	breq no_timeout  ; saturation check	
	inc  STEP_PERIOD

no_timeout:
	nop 
.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Step timing control for step multiplication mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro OVERSTEP_TIMER_COUNTER_CHECK 
		
	sbis TIFR1,OCF1A
	rjmp no_timeout	

	; Ok timeout has occured, clear the timeout flag
	sbi  TIFR1,OCF1A

	; include these cycles in counting real step period
	; no saturation cotrol as these cycles occur after the counter is cleared
	inc  STEP_PERIOD

	; Check step counting
	inc  OVERSTEP_MUL_CNTR
	ldi  temp2,STEP_MULTIPLIER_RATIO
	cp   OVERSTEP_MUL_CNTR,temp2
	breq all_steps_executed

execute_next_step:
	ldi  temp,1 	
	rjmp  end_result
	 	  	
all_steps_executed:
	clr   OVERSTEP_MUL_CNTR
	ldi	  temp,2
	rjmp  end_result			
	
no_timeout:
	clr temp
end_result:
	nop
.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Step time period threshold calculations 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.macro OVERSTEP_PROCESS_STEP

	; compare with upper threshold 	
	ldi temp, STEP_PERIOD_UP_THR
	cp 	STEP_PERIOD, temp
	brlo compare_lower
	;ok period longer then upper threshold
	inc OVER_TIMEOUT_CNTR_UP
compare_lower:
	ldi temp, STEP_PERIOD_DN_THR
	cp 	STEP_PERIOD, temp
	brge process_step
	;ok period shorter then lower threshold
	inc OVER_TIMEOUT_CNTR_DN
process_step:
	clr STEP_PERIOD
	; Reset the timer
	clr  temp
	sts  TCNT1H,temp
	sts  TCNT1L,temp
	sbi  TIFR1,OCF1A

.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Step mode (mul/div) selection
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro OVERSTEP_CHECK_MODE
	ldi temp,STEP_MULTIPLIED_MODE
	cp STEP_MODE,temp
	brne current_div_mode

current_mul_mode:
	ldi temp,STEP_RATIO
	cp  temp,OVER_TIMEOUT_CNTR_DN
	brne checked 
	; Ok, all step pulses are shorter then threshold, change the mode
	ldi STEP_MODE,STEP_DIVIDED_MODE
	rjmp checked 

current_div_mode:

	ldi temp,STEP_RATIO
	cp  temp,OVER_TIMEOUT_CNTR_UP
	brne checked 
	; Ok, all step pulses are longer then threshold, change the mode
	ldi STEP_MODE,STEP_MULTIPLIED_MODE
	
checked:	
	clr OVER_TIMEOUT_CNTR_UP
	clr OVER_TIMEOUT_CNTR_DN
.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Step timing control for step division mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro OVERSTEP_STEP_DIV_CHECK	
	 ; check division ratio of real steps
	 ; returns non-zero in 'temp' for succesful check
	 inc OVERSTEP_DIV_CNTR
	 ldi temp2,STEP_DIVIDER_RATIO
	 cp OVERSTEP_DIV_CNTR,temp2
	 breq  next_real_step
	 clr   temp
	 rjmp  end_result 	  	
next_real_step:
	 clr   OVERSTEP_DIV_CNTR
	 ser   temp
end_result:
	 nop
.endmacro




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Step control for embedded half step mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.macro TRANSFER
	ldi temp,@1
	mov @0, temp
.endmacro

.macro OVERSTEP_HALFSTEP_STEP
	clr  STEP_HOME
	SBIC_DIR
	rjmp left_dir_load
right_dir_load:
	inc  OVERSTEP_HALFSTEP_INDEX
	andi OVERSTEP_HALFSTEP_INDEX,0x0
	rjmp choose_step
left_dir_load:
	dec  OVERSTEP_HALFSTEP_INDEX
	andi OVERSTEP_HALFSTEP_INDEX,0x07
	rjmp choose_step
choose_step:
	cpi  OVERSTEP_HALFSTEP_INDEX,7
	brne check_6
	rjmp step_7
check_6:
	cpi  OVERSTEP_HALFSTEP_INDEX,6
	brne check_5
	rjmp step_6
check_5:
	cpi  OVERSTEP_HALFSTEP_INDEX,5
	brne check_4 
	rjmp step_5
check_4:
	cpi  OVERSTEP_HALFSTEP_INDEX,4
	brne check_3 
	rjmp step_4
check_3:
	cpi  OVERSTEP_HALFSTEP_INDEX,3
	brne check_2
	rjmp step_3
check_2:
	cpi  OVERSTEP_HALFSTEP_INDEX,2
	brne check_1
	rjmp step_2
check_1:
	cpi  OVERSTEP_HALFSTEP_INDEX,1
	brne step_0 
	rjmp step_1

step_0:
	ser STEP_HOME
	TRANSFER CH_A_SPI_L,    LO(SIN_00)
	TRANSFER CH_A_SPI_H,   (HI(SIN_00)|CH_A_SPI_H_CTRL)
	TRANSFER CH_B_SPI_L,    LO(SIN_90)
	TRANSFER CH_B_SPI_H,   (HI(SIN_90)|CH_B_SPI_H_CTRL)
	TRANSFER CH_A_B_PHASE, STEP_0
	rjmp step_end 

step_1:
	TRANSFER CH_A_SPI_L,    LO(SIN_45)
	TRANSFER CH_A_SPI_H,   (HI(SIN_45)|CH_A_SPI_H_CTRL)
	TRANSFER CH_B_SPI_L,    LO(SIN_45)
	TRANSFER CH_B_SPI_H,   (HI(SIN_45)|CH_B_SPI_H_CTRL)
	TRANSFER CH_A_B_PHASE, STEP_1
	rjmp step_end 

step_2:
	TRANSFER CH_A_SPI_L,    LO(SIN_90)
	TRANSFER CH_A_SPI_H,   (HI(SIN_90)|CH_A_SPI_H_CTRL)
	TRANSFER CH_B_SPI_L,    LO(SIN_00)
	TRANSFER CH_B_SPI_H,   (HI(SIN_00)|CH_B_SPI_H_CTRL)
	TRANSFER CH_A_B_PHASE, STEP_2
	rjmp step_end 

step_3:
	TRANSFER CH_A_SPI_L,    LO(SIN_45)
	TRANSFER CH_A_SPI_H,   (HI(SIN_45)|CH_A_SPI_H_CTRL)
	TRANSFER CH_B_SPI_L,    LO(SIN_45)
	TRANSFER CH_B_SPI_H,   (HI(SIN_45)|CH_B_SPI_H_CTRL)
	TRANSFER CH_A_B_PHASE, STEP_3
	rjmp step_end 

step_4:
	TRANSFER CH_A_SPI_L,    LO(SIN_00)
	TRANSFER CH_A_SPI_H,   (HI(SIN_00)|CH_A_SPI_H_CTRL)
	TRANSFER CH_B_SPI_L,    LO(SIN_90)
	TRANSFER CH_B_SPI_H,   (HI(SIN_90)|CH_B_SPI_H_CTRL)
	TRANSFER CH_A_B_PHASE, STEP_4
	rjmp step_end 

step_5:
	TRANSFER CH_A_SPI_L,    LO(SIN_45)
	TRANSFER CH_A_SPI_H,   (HI(SIN_45)|CH_A_SPI_H_CTRL)
	TRANSFER CH_B_SPI_L,    LO(SIN_45)
	TRANSFER CH_B_SPI_H,   (HI(SIN_45)|CH_B_SPI_H_CTRL)
	TRANSFER CH_A_B_PHASE, STEP_5
	rjmp step_end 

step_6:
	TRANSFER CH_A_SPI_L,    LO(SIN_90)
	TRANSFER CH_A_SPI_H,   (HI(SIN_90)|CH_A_SPI_H_CTRL)
	TRANSFER CH_B_SPI_L,    LO(SIN_00)
	TRANSFER CH_B_SPI_H,   (HI(SIN_00)|CH_B_SPI_H_CTRL)
	TRANSFER CH_A_B_PHASE, STEP_6
	rjmp step_end 

step_7:
	TRANSFER CH_A_SPI_L,    LO(SIN_45) 
	TRANSFER CH_A_SPI_H,   (HI(SIN_45)|CH_A_SPI_H_CTRL)
	TRANSFER CH_B_SPI_L,    LO(SIN_45)
	TRANSFER CH_B_SPI_H,   (HI(SIN_45)|CH_B_SPI_H_CTRL)
	TRANSFER CH_A_B_PHASE, STEP_7

step_end:
	nop
.endmacro








