;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ******************** Global settings ******************************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#define HW_MCK	20000000

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Global register allocation 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.def OVER_TIMEOUT_CNTR_UP	=   r0
.def OVER_TIMEOUT_CNTR_DN	=   r1
.def STEP_PERIOD			=   r2
.def OVERSTEP_DIV_CNTR		=   r3
.def OVERSTEP_MUL_CNTR		=   r4
.def ENABLE_MODE			=   r5
.def CURR_LIMIT_TOUT_L		=   r6
.def CURR_LIMIT_TOUT_H 		=   r7				
.def CH_A_SPI_L    			=	r8
.def CH_A_SPI_H	   			=	r9
.def CH_B_SPI_L   			=	r10
.def CH_B_SPI_H	   			=	r11  
.def CH_A_B_PHASE  			=	r12



.def temp 		   			 = 	r16
.def temp2 		   			 =	r17

.def SPI_A_CTRL	   			 =	r18
.def SPI_B_CTRL	   			 =	r19
.def STEP_STATUS   			 =	r20
.def DECAY_STATUS  			 =  r21	
.def OVERSTEP_HALFSTEP_INDEX =  r22
.def STEP_HOME				 =  r23
.def STEP_MODE			     =  r24
.def IDLE_STATUS  			 =	r25


;
; Used for indirect addressing
;
; R26 - R27 -> X register 
; R28 - R29 -> Y register 
; R30 - R31 -> Z register 



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; **************** Commonly used macros and definitions *************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Generic 

#define _BV(x) (1<<x)
#define LO(x_) ( (x_) & 0xff)
#define HI(x_) ( ((x_)>>8) & 0xff)


; Step table bit definitions


#define CH_A_PATH_1_BIT		7
#define CH_B_PATH_1_BIT		6
#define CH_A_SLOW_BIT		5
#define CH_B_SLOW_BIT		4


; Table boundary constants
#define CH_AB_NONE			0
#define CH_A_MIN			1
#define CH_A_MAX			2
#define CH_B_MIN			3
#define CH_B_MAX			4
#define CH_A_MIN_MIN		5
#define CH_A_MAX_MAX		6


; A/D converter definitions
#define CH_A_SPI_H_DATA_MASK	0x0f
#define CH_A_SPI_H_CTRL			0x20

#define CH_B_SPI_H_DATA_MASK	0x0f
#define CH_B_SPI_H_CTRL			0x30

#define CH_A_SPI_H_FULL_STEP_CTRL	0x2f
#define CH_A_SPI_L_FULL_STEP_CTRL	0xff

#define CH_B_SPI_H_FULL_STEP_CTRL	0x3f
#define CH_B_SPI_L_FULL_STEP_CTRL	0xff

; Step modes

#define STEP_MULTIPLIED_MODE		0x0
#define STEP_DIVIDED_MODE			0x1



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; **************** Some global system macros ************************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Watchdog initialization 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro WATCHDOG_INIT_FAST
	; Initialize to shortest possible time (16ms)
	wdr
	ldi	temp,_BV(WDCE)|_BV(WDE)
	sts WDTCSR,temp  
.endmacro

.macro WATCHDOG_INIT_SLOW
	; Initialize to long timeout (0.5s)
	wdr
	ldi	temp,_BV(WDCE)|_BV(WDE)|_BV(WDP2)|_BV(WDP0)
	sts WDTCSR,temp  
.endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Stack initialization 
;
; Note : The stack is not used in the program 
;		 This is for  no reason :-)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.macro STACK_INIT
    ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
.endmacro
