;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ******************** Table data processing ************************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Initialization of controller settings 
; (from automatically generated data)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro ROM_TABLE_INIT

	; Copy ROM sinus table to RAM
	; Note - address conversion needed (ROM  - word size, RAM - byte size )

	ldi ZL, low(rom_sinus_table_step << 1)
	ldi ZH, high(rom_sinus_table_step << 1)	

	ldi YL, low(rom_sinus_table_step_end << 1)
	ldi YH, high(rom_sinus_table_step_end << 1)	

	ldi XL, low(sinus_table_step)
	ldi XH, high(sinus_table_step)

loop_sinus:

	lpm temp,Z+		; Copy whole word at once
	st  X+,temp
	lpm temp,Z+
	st  X+,temp

	cp	ZL,YL
	brne loop_sinus
	cp	ZH,YH
	brne loop_sinus

	; Copy ROM decay table to RAM

	ldi ZL, low(rom_decay_table << 1)
	ldi ZH, high(rom_decay_table << 1)	

	ldi YL, low(rom_decay_table_end << 1)
	ldi YH, high(rom_decay_table_end << 1)	

	ldi XL, low(decay_table)
	ldi XH, high(decay_table)

loop_decay:
	
	lpm temp,Z+ ; read of low  byte - decay data
	st  X+,temp
	lpm temp,Z+	; read of high byte - currently dummy data

	cp	ZL,YL
	brne loop_decay
	cp	ZH,YH
	brne loop_decay

	; Initialize pointers

	ldi XL, low(sinus_table_step)  		; Bridge A pointers
	ldi XH, high(sinus_table_step)	

	ldi YL, low(sinus_table_step_half)		; Bridge B pointers
	ldi YH, high(sinus_table_step_half)	

	ldi ZL, low(decay_table)			; Common A/B control words
	ldi ZH, high(decay_table)

	; Initialize D/A control part not present in sinus tables

	ldi  SPI_A_CTRL,CH_A_SPI_H_CTRL
	ldi  SPI_B_CTRL,CH_B_SPI_H_CTRL

	; Load initial value of control word

	ld 	 CH_A_B_PHASE,Z

	; Set initial home position
	
	ser  STEP_HOME

.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Loading single step data and adjusting table pointers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro ROM_TABLE_STEP
	
	SBIC_DIR
	rjmp left_dir_load

right_dir_load:

	mov  temp,CH_A_B_PHASE
	andi temp ,0x07				; extract boundary info
	breq end_right_dir_prepare	; no boudary condition - skip further checks

	; Fix table pointers if needed

check_a_max_max:
	cpi temp,CH_A_MAX_MAX
	brne check_a_max
	ldi ZL, low(decay_table)	
	ldi ZH, high(decay_table)
	rjmp a_max					; common condition
check_a_max:
	cpi temp,CH_A_MAX
	brne check_b_max	
a_max:
	ldi XL, low(sinus_table_step)
	ldi XH, high(sinus_table_step)	
	rjmp end_right_dir_prepare

check_b_max:

	cpi temp,CH_B_MAX
	brne end_right_dir_prepare
	ldi YL, low(sinus_table_step)
	ldi YH, high(sinus_table_step)	

end_right_dir_prepare:

	ld CH_A_SPI_L,X+
	ld CH_A_SPI_H,X+
	or CH_A_SPI_H,SPI_A_CTRL
	ld CH_B_SPI_L,Y+
	ld CH_B_SPI_H,Y+
	or CH_B_SPI_H,SPI_B_CTRL
	ld CH_A_B_PHASE,Z+
	
	rjmp end_dir_load

left_dir_load:

	mov  temp,CH_A_B_PHASE
	andi temp ,0x07				; extract boundary info
	breq end_left_dir_prepare	; no boudary condition - skip further checks

	; Fix table pointers if needed

check_a_min_min:
	cpi temp,CH_A_MIN_MIN
	brne check_a_min	
	ldi ZL, low(decay_table_end)	
	ldi ZH, high(decay_table_end)
	rjmp a_min
check_a_min:
	cpi temp,CH_A_MIN
	brne check_b_min	
a_min:
	ldi XL, low(sinus_table_step_end)
	ldi XH, high(sinus_table_step_end)	
	rjmp end_left_dir_prepare

check_b_min:
	cpi temp,CH_B_MIN
	brne end_left_dir_prepare
	ldi YL, low(sinus_table_step_end)
	ldi YH, high(sinus_table_step_end)	

end_left_dir_prepare:

	ld CH_A_B_PHASE,-Z
	ld CH_B_SPI_H,-Y
	or CH_B_SPI_H,SPI_B_CTRL
	ld CH_B_SPI_L,-Y
	ld CH_A_SPI_H,-X
	or CH_A_SPI_H,SPI_A_CTRL
	ld CH_A_SPI_L,-X

end_dir_load:

	; check if home position is reached
	clr  STEP_HOME	
	mov  temp,CH_A_B_PHASE
	andi temp ,0x07
	cpi  temp,CH_A_MIN_MIN
	brne end_next_step 
	ser  STEP_HOME

end_next_step:

.endmacro
